// src/discordjs-react.tsx
import { Events } from "discord.js";
import React2 from "react";

// src/renderer.tsx
import { ComponentType } from "discord.js";
import React from "react";
import { concatMap, Subject } from "rxjs";

// src/container.ts
var Container = class {
  items = [];
  add(...items) {
    this.items.push(...items);
  }
  addBefore(item, before) {
    let index = this.items.indexOf(before);
    if (index === -1) {
      index = this.items.length;
    }
    this.items.splice(index, 0, item);
  }
  remove(toRemove) {
    this.items = this.items.filter((item) => item !== toRemove);
  }
  clear() {
    this.items = [];
  }
  find(predicate) {
    return this.items.find(predicate);
  }
  findType(type) {
    for (const item of this.items) {
      if (item instanceof type)
        return item;
    }
  }
  [Symbol.iterator]() {
    return this.items[Symbol.iterator]();
  }
};

// src/helpers/helpers.ts
function omit(subject, keys) {
  const result = {};
  for (const key in subject) {
    if (!keys.includes(key)) {
      result[key] = subject[key];
    }
  }
  return result;
}
function toError(value) {
  return value instanceof Error ? value : new Error(String(value));
}
function raise(error) {
  throw toError(error);
}
function last(array) {
  return array[array.length - 1];
}
var isInstanceOf = (Constructor) => (value) => value instanceof Constructor;

// src/reconciler.ts
import ReactReconciler from "react-reconciler";
import { DefaultEventPriority } from "react-reconciler/constants";

// src/node.ts
var Node = class {
  constructor(props) {
    this.props = props;
  }
  children = new Container();
  interaction = void 0;
  onComplete = void 0;
  getActionRowItemData() {
    throw new Error("Not implemented");
  }
  modifyMessageOptions(options) {
  }
  handleComponentInteraction(interaction, onComplete) {
    return void 0;
  }
  completeInteraction() {
    this.interaction = void 0;
    if (this.onComplete)
      this.onComplete();
  }
  get isDeferred() {
    var _a;
    return (_a = this.interaction) == null ? void 0 : _a.deferred;
  }
  get text() {
    return [...this.children].map((child) => child.text).join("");
  }
};

// src/components/text-node.ts
var TextNode = class extends Node {
  modifyMessageOptions(options) {
    options.content = options.content + this.props;
  }
  get text() {
    return this.props;
  }
};

// src/reconciler.ts
var config = {
  supportsMutation: true,
  supportsPersistence: false,
  supportsHydration: false,
  isPrimaryRenderer: true,
  scheduleTimeout: global.setTimeout,
  cancelTimeout: global.clearTimeout,
  noTimeout: -1,
  // eslint-disable-next-line unicorn/no-null
  getRootHostContext: () => null,
  getChildHostContext: (parentContext) => parentContext,
  createInstance: (type, props) => {
    if (type !== "reacord-element") {
      raise(`Unknown element type: ${type}`);
    }
    if (typeof props.createNode !== "function") {
      raise(`Missing createNode function`);
    }
    const node = props.createNode(props.props);
    if (!(node instanceof Node)) {
      raise(`createNode function did not return a Node`);
    }
    return node;
  },
  createTextInstance: (text) => new TextNode(text),
  shouldSetTextContent: () => false,
  detachDeletedInstance: (instance) => {
  },
  beforeActiveInstanceBlur: () => {
  },
  afterActiveInstanceBlur: () => {
  },
  // eslint-disable-next-line unicorn/no-null
  getInstanceFromNode: (node) => null,
  // eslint-disable-next-line unicorn/no-null
  getInstanceFromScope: (scopeInstance) => null,
  clearContainer: (renderer) => {
    renderer.nodes.clear();
  },
  appendChildToContainer: (renderer, child) => {
    renderer.nodes.add(child);
  },
  removeChildFromContainer: (renderer, child) => {
    renderer.nodes.remove(child);
  },
  insertInContainerBefore: (renderer, child, before) => {
    renderer.nodes.addBefore(child, before);
  },
  appendInitialChild: (parent, child) => {
    parent.children.add(child);
  },
  appendChild: (parent, child) => {
    parent.children.add(child);
  },
  removeChild: (parent, child) => {
    parent.children.remove(child);
  },
  insertBefore: (parent, child, before) => {
    parent.children.addBefore(child, before);
  },
  prepareUpdate: () => true,
  commitUpdate: (node, payload, type, oldProps, newProps) => {
    node.props = newProps.props;
  },
  commitTextUpdate: (node, oldText, newText) => {
    node.props = newText;
  },
  // eslint-disable-next-line unicorn/no-null
  prepareForCommit: () => null,
  resetAfterCommit: (renderer) => {
    renderer.render();
  },
  prepareScopeUpdate: (scopeInstance, instance) => {
  },
  preparePortalMount: () => raise("Portals are not supported"),
  getPublicInstance: () => raise("Refs are currently not supported"),
  finalizeInitialChildren: () => false,
  getCurrentEventPriority: () => DefaultEventPriority
};
var reconciler = ReactReconciler(config);

// src/message.ts
var DiscordJSReactMessage = class {
  constructor(raw, renderer) {
    this.raw = raw;
    this.renderer = renderer;
  }
};

// src/renderer.tsx
import { randomUUID } from "crypto";
function getNextActionRow(options) {
  let actionRow = last(options.actionRows);
  const firstItem = actionRow == null ? void 0 : actionRow[0];
  if (actionRow == void 0 || actionRow.length >= 5 || firstItem && "type" in firstItem && firstItem.type === ComponentType.StringSelect) {
    actionRow = [];
    options.actionRows.push(actionRow);
  }
  return actionRow;
}
var LOADING_EMOJI = "<a:loading:1081524604419453028>";
var InstanceContext = React.createContext(void 0);
function useInstance() {
  const instance = React.useContext(InstanceContext);
  if (instance == void 0) {
    throw new Error("Attempted to useInstance outside of a renderer");
  }
  return instance;
}
var Renderer = class {
  constructor(options, discordJSReact, initialContent) {
    this.options = options;
    this.discordJSReact = discordJSReact;
    const container = reconciler.createContainer(
      this,
      0,
      // eslint-disable-next-line unicorn/no-null
      null,
      false,
      // eslint-disable-next-line unicorn/no-null
      null,
      "reacord",
      () => {
      },
      // eslint-disable-next-line unicorn/no-null
      null
    );
    reconciler.updateContainer(
      /* @__PURE__ */ React.createElement(InstanceContext.Provider, { value: this }, /* @__PURE__ */ React.createElement(discordJSReact.config.wrapper, null, initialContent)),
      container
    );
  }
  nodes = new Container();
  message;
  active = true;
  updates = new Subject();
  rendererId = randomUUID();
  updateSubscription = this.updates.pipe(concatMap((payload) => this.updateMessage(payload))).subscribe({ error: console.error });
  render() {
    if (!this.active) {
      console.warn("Attempted to update a deactivated message");
      return;
    }
    this.updates.next({
      options: this.getMessageOptions(),
      action: "stateChange"
    });
  }
  deactivate() {
    this.active = false;
    this.updates.next({
      options: this.getMessageOptions(),
      action: "deactivate"
    });
  }
  destroy() {
    this.active = false;
    this.updates.next({ action: "destroy" });
  }
  handleComponentInteraction(interaction) {
    for (const node of this.nodes) {
      const handler = node.handleComponentInteraction(interaction, () => {
        this.updates.next({
          options: this.getMessageOptions(),
          action: "interactionComplete",
          interaction
        });
      });
      if (handler) {
        setTimeout(() => {
          this.updates.next({ action: "deferUpdate", interaction, node: handler, renderer: this });
        }, 2e3);
        return handler;
      }
    }
    return void 0;
  }
  getMessageOptions() {
    const options = {
      content: "",
      embeds: [],
      actionRows: []
    };
    for (const node of this.nodes) {
      node.modifyMessageOptions(options);
    }
    return {
      components: options.actionRows.map((row) => ({
        type: ComponentType.ActionRow,
        components: row
      })),
      content: options.content,
      embeds: options.embeds
    };
  }
  trackMessage(message) {
    this.message = new DiscordJSReactMessage(message, this);
  }
  async updateMessage(payload) {
    var _a, _b;
    if (payload.action === "destroy") {
      this.updateSubscription.unsubscribe();
      await ((_a = this.message) == null ? void 0 : _a.raw.delete());
      return;
    }
    if (payload.action === "deactivate") {
      this.updateSubscription.unsubscribe();
      await ((_b = this.message) == null ? void 0 : _b.raw.edit({
        ...payload.options,
        // TODO: Disable all components
        components: []
      }));
      return;
    }
    if (payload.action === "deferUpdate") {
      if (payload.interaction.deferred || payload.interaction.replied)
        return;
      await payload.interaction.deferUpdate();
      payload.renderer.render();
      return;
    }
    if (payload.action === "stateChange" && this.message) {
      if (this.options.type === "interaction") {
        await this.options.interaction.editReply(payload.options);
      } else {
        await this.message.raw.edit(payload.options);
      }
      return;
    }
    if (payload.action === "interactionComplete") {
      if (this.discordJSReact.renderers.find((r) => r.options.type === "interaction" && r.options.interaction.id === payload.interaction.id))
        return;
      if (payload.interaction.replied) {
        return;
      }
      if (payload.interaction.deferred) {
        await payload.interaction.editReply(payload.options);
        return;
      } else {
        await payload.interaction.update(payload.options);
      }
      return;
    }
    if (this.options.type === "interaction") {
      const intr = this.options.interaction;
      if (intr.deferred || intr.replied) {
        await intr.editReply(payload.options);
        return;
      }
      const created = await intr.reply({
        ephemeral: this.options.ephemeral,
        fetchReply: true,
        ...payload.options
      });
      this.trackMessage(created);
    }
  }
};

// src/discordjs-react.tsx
var DiscordJSReact = class {
  constructor(client, config2 = {
    wrapper: (props) => /* @__PURE__ */ React2.createElement(React2.Fragment, null, props.children)
  }) {
    this.client = client;
    this.config = config2;
    client.on(Events.InteractionCreate, (interaction) => {
      if (!interaction.isMessageComponent())
        return;
      for (const renderer of this.renderers) {
        renderer.handleComponentInteraction(interaction);
      }
    });
  }
  // Maps the interaction id to the renderer
  renderers = [];
  deactivate(renderer) {
    this.renderers = this.renderers.filter((it) => it !== renderer);
    renderer.deactivate();
  }
  get maxInstances() {
    return this.config.maxInstances ?? 50;
  }
  createRenderer(renderer, initialContent) {
    return new Renderer(renderer, this, initialContent);
  }
  activateRenderer(renderer) {
    if (this.renderers.length >= this.maxInstances) {
      this.deactivate(this.renderers[0]);
    }
    this.renderers.push(renderer);
    return renderer;
  }
  ephemeralReply(interaction, initialContent) {
    return this.activateRenderer(
      this.createRenderer(
        {
          type: "interaction",
          interaction,
          ephemeral: true
        },
        initialContent
      )
    );
  }
  getMessage(index = 0) {
    var _a;
    return (_a = this.renderers.at(index)) == null ? void 0 : _a.message;
  }
};

// src/components/action-row.tsx
import React4 from "react";

// src/element.ts
import React3 from "react";
function DiscordJSReactElement(props) {
  return React3.createElement("reacord-element", props);
}

// src/components/action-row.tsx
function ActionRow2(props) {
  return /* @__PURE__ */ React4.createElement(DiscordJSReactElement, { props, createNode: () => new ActionRowNode(props) }, props.children);
}
var ActionRowNode = class extends Node {
  modifyMessageOptions(options) {
    options.actionRows.push([]);
    for (const child of this.children) {
      child.modifyMessageOptions(options);
    }
  }
  handleComponentInteraction(interaction, onComplete) {
    for (const child of this.children) {
      const handler = child.handleComponentInteraction(interaction, onComplete);
      if (handler) {
        return handler;
      }
    }
    return void 0;
  }
};

// src/components/button.tsx
import { ButtonStyle, ComponentType as ComponentType2 } from "discord.js";
import { randomUUID as randomUUID2 } from "node:crypto";
import React5 from "react";
function Button(props) {
  return /* @__PURE__ */ React5.createElement(DiscordJSReactElement, { props, createNode: () => new ButtonNode(props) }, /* @__PURE__ */ React5.createElement(DiscordJSReactElement, { props: {}, createNode: () => new ButtonLabelNode({}) }, props.label));
}
var ButtonNode = class extends Node {
  customId = randomUUID2();
  // this has text children, but buttons themselves shouldn't yield text
  // eslint-disable-next-line class-methods-use-this
  get text() {
    return "";
  }
  getActionRowItemData() {
    var _a;
    const buttonStyle = this.props.style;
    const opts = {
      type: ComponentType2.Button,
      customId: this.customId,
      style: buttonStyle ? ButtonStyle[buttonStyle] : ButtonStyle.Secondary,
      disabled: this.props.disabled || this.isDeferred !== void 0,
      emoji: this.isDeferred ? LOADING_EMOJI : this.props.emoji,
      label: (_a = this.children.findType(ButtonLabelNode)) == null ? void 0 : _a.text
    };
    return opts;
  }
  modifyMessageOptions(options) {
    getNextActionRow(options).push(
      this.getActionRowItemData()
    );
  }
  handleComponentInteraction(interaction, onComplete) {
    const shouldHandleInteraction = interaction.isButton() && interaction.customId === this.customId;
    if (!shouldHandleInteraction)
      return void 0;
    this.interaction = interaction;
    this.onComplete = onComplete;
    Promise.resolve(this.props.onClick(interaction)).then(
      () => this.completeInteraction()
    ).catch(
      (err) => {
        throw err;
      }
    );
    return this;
  }
};
var ButtonLabelNode = class extends Node {
};

// src/components/embeds/embed.tsx
import React6 from "react";

// src/components/embeds/embed-child.ts
var EmbedChildNode = class extends Node {
};

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_freeGlobal.js
var freeGlobal = typeof global == "object" && global && global.Object === Object && global;
var freeGlobal_default = freeGlobal;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_root.js
var freeSelf = typeof self == "object" && self && self.Object === Object && self;
var root = freeGlobal_default || freeSelf || Function("return this")();
var root_default = root;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_Symbol.js
var Symbol2 = root_default.Symbol;
var Symbol_default = Symbol2;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_getRawTag.js
var objectProto = Object.prototype;
var hasOwnProperty = objectProto.hasOwnProperty;
var nativeObjectToString = objectProto.toString;
var symToStringTag = Symbol_default ? Symbol_default.toStringTag : void 0;
function getRawTag(value) {
  var isOwn = hasOwnProperty.call(value, symToStringTag), tag = value[symToStringTag];
  try {
    value[symToStringTag] = void 0;
    var unmasked = true;
  } catch (e) {
  }
  var result = nativeObjectToString.call(value);
  if (unmasked) {
    if (isOwn) {
      value[symToStringTag] = tag;
    } else {
      delete value[symToStringTag];
    }
  }
  return result;
}
var getRawTag_default = getRawTag;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_objectToString.js
var objectProto2 = Object.prototype;
var nativeObjectToString2 = objectProto2.toString;
function objectToString(value) {
  return nativeObjectToString2.call(value);
}
var objectToString_default = objectToString;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_baseGetTag.js
var nullTag = "[object Null]";
var undefinedTag = "[object Undefined]";
var symToStringTag2 = Symbol_default ? Symbol_default.toStringTag : void 0;
function baseGetTag(value) {
  if (value == null) {
    return value === void 0 ? undefinedTag : nullTag;
  }
  return symToStringTag2 && symToStringTag2 in Object(value) ? getRawTag_default(value) : objectToString_default(value);
}
var baseGetTag_default = baseGetTag;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/isObjectLike.js
function isObjectLike(value) {
  return value != null && typeof value == "object";
}
var isObjectLike_default = isObjectLike;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/isSymbol.js
var symbolTag = "[object Symbol]";
function isSymbol(value) {
  return typeof value == "symbol" || isObjectLike_default(value) && baseGetTag_default(value) == symbolTag;
}
var isSymbol_default = isSymbol;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_arrayMap.js
function arrayMap(array, iteratee) {
  var index = -1, length = array == null ? 0 : array.length, result = Array(length);
  while (++index < length) {
    result[index] = iteratee(array[index], index, array);
  }
  return result;
}
var arrayMap_default = arrayMap;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/isArray.js
var isArray = Array.isArray;
var isArray_default = isArray;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_baseToString.js
var INFINITY = 1 / 0;
var symbolProto = Symbol_default ? Symbol_default.prototype : void 0;
var symbolToString = symbolProto ? symbolProto.toString : void 0;
function baseToString(value) {
  if (typeof value == "string") {
    return value;
  }
  if (isArray_default(value)) {
    return arrayMap_default(value, baseToString) + "";
  }
  if (isSymbol_default(value)) {
    return symbolToString ? symbolToString.call(value) : "";
  }
  var result = value + "";
  return result == "0" && 1 / value == -INFINITY ? "-0" : result;
}
var baseToString_default = baseToString;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/isObject.js
function isObject(value) {
  var type = typeof value;
  return value != null && (type == "object" || type == "function");
}
var isObject_default = isObject;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/toString.js
function toString(value) {
  return value == null ? "" : baseToString_default(value);
}
var toString_default = toString;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_arrayReduce.js
function arrayReduce(array, iteratee, accumulator, initAccum) {
  var index = -1, length = array == null ? 0 : array.length;
  if (initAccum && length) {
    accumulator = array[++index];
  }
  while (++index < length) {
    accumulator = iteratee(accumulator, array[index], index, array);
  }
  return accumulator;
}
var arrayReduce_default = arrayReduce;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_basePropertyOf.js
function basePropertyOf(object) {
  return function(key) {
    return object == null ? void 0 : object[key];
  };
}
var basePropertyOf_default = basePropertyOf;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_deburrLetter.js
var deburredLetters = {
  // Latin-1 Supplement block.
  "\xC0": "A",
  "\xC1": "A",
  "\xC2": "A",
  "\xC3": "A",
  "\xC4": "A",
  "\xC5": "A",
  "\xE0": "a",
  "\xE1": "a",
  "\xE2": "a",
  "\xE3": "a",
  "\xE4": "a",
  "\xE5": "a",
  "\xC7": "C",
  "\xE7": "c",
  "\xD0": "D",
  "\xF0": "d",
  "\xC8": "E",
  "\xC9": "E",
  "\xCA": "E",
  "\xCB": "E",
  "\xE8": "e",
  "\xE9": "e",
  "\xEA": "e",
  "\xEB": "e",
  "\xCC": "I",
  "\xCD": "I",
  "\xCE": "I",
  "\xCF": "I",
  "\xEC": "i",
  "\xED": "i",
  "\xEE": "i",
  "\xEF": "i",
  "\xD1": "N",
  "\xF1": "n",
  "\xD2": "O",
  "\xD3": "O",
  "\xD4": "O",
  "\xD5": "O",
  "\xD6": "O",
  "\xD8": "O",
  "\xF2": "o",
  "\xF3": "o",
  "\xF4": "o",
  "\xF5": "o",
  "\xF6": "o",
  "\xF8": "o",
  "\xD9": "U",
  "\xDA": "U",
  "\xDB": "U",
  "\xDC": "U",
  "\xF9": "u",
  "\xFA": "u",
  "\xFB": "u",
  "\xFC": "u",
  "\xDD": "Y",
  "\xFD": "y",
  "\xFF": "y",
  "\xC6": "Ae",
  "\xE6": "ae",
  "\xDE": "Th",
  "\xFE": "th",
  "\xDF": "ss",
  // Latin Extended-A block.
  "\u0100": "A",
  "\u0102": "A",
  "\u0104": "A",
  "\u0101": "a",
  "\u0103": "a",
  "\u0105": "a",
  "\u0106": "C",
  "\u0108": "C",
  "\u010A": "C",
  "\u010C": "C",
  "\u0107": "c",
  "\u0109": "c",
  "\u010B": "c",
  "\u010D": "c",
  "\u010E": "D",
  "\u0110": "D",
  "\u010F": "d",
  "\u0111": "d",
  "\u0112": "E",
  "\u0114": "E",
  "\u0116": "E",
  "\u0118": "E",
  "\u011A": "E",
  "\u0113": "e",
  "\u0115": "e",
  "\u0117": "e",
  "\u0119": "e",
  "\u011B": "e",
  "\u011C": "G",
  "\u011E": "G",
  "\u0120": "G",
  "\u0122": "G",
  "\u011D": "g",
  "\u011F": "g",
  "\u0121": "g",
  "\u0123": "g",
  "\u0124": "H",
  "\u0126": "H",
  "\u0125": "h",
  "\u0127": "h",
  "\u0128": "I",
  "\u012A": "I",
  "\u012C": "I",
  "\u012E": "I",
  "\u0130": "I",
  "\u0129": "i",
  "\u012B": "i",
  "\u012D": "i",
  "\u012F": "i",
  "\u0131": "i",
  "\u0134": "J",
  "\u0135": "j",
  "\u0136": "K",
  "\u0137": "k",
  "\u0138": "k",
  "\u0139": "L",
  "\u013B": "L",
  "\u013D": "L",
  "\u013F": "L",
  "\u0141": "L",
  "\u013A": "l",
  "\u013C": "l",
  "\u013E": "l",
  "\u0140": "l",
  "\u0142": "l",
  "\u0143": "N",
  "\u0145": "N",
  "\u0147": "N",
  "\u014A": "N",
  "\u0144": "n",
  "\u0146": "n",
  "\u0148": "n",
  "\u014B": "n",
  "\u014C": "O",
  "\u014E": "O",
  "\u0150": "O",
  "\u014D": "o",
  "\u014F": "o",
  "\u0151": "o",
  "\u0154": "R",
  "\u0156": "R",
  "\u0158": "R",
  "\u0155": "r",
  "\u0157": "r",
  "\u0159": "r",
  "\u015A": "S",
  "\u015C": "S",
  "\u015E": "S",
  "\u0160": "S",
  "\u015B": "s",
  "\u015D": "s",
  "\u015F": "s",
  "\u0161": "s",
  "\u0162": "T",
  "\u0164": "T",
  "\u0166": "T",
  "\u0163": "t",
  "\u0165": "t",
  "\u0167": "t",
  "\u0168": "U",
  "\u016A": "U",
  "\u016C": "U",
  "\u016E": "U",
  "\u0170": "U",
  "\u0172": "U",
  "\u0169": "u",
  "\u016B": "u",
  "\u016D": "u",
  "\u016F": "u",
  "\u0171": "u",
  "\u0173": "u",
  "\u0174": "W",
  "\u0175": "w",
  "\u0176": "Y",
  "\u0177": "y",
  "\u0178": "Y",
  "\u0179": "Z",
  "\u017B": "Z",
  "\u017D": "Z",
  "\u017A": "z",
  "\u017C": "z",
  "\u017E": "z",
  "\u0132": "IJ",
  "\u0133": "ij",
  "\u0152": "Oe",
  "\u0153": "oe",
  "\u0149": "'n",
  "\u017F": "s"
};
var deburrLetter = basePropertyOf_default(deburredLetters);
var deburrLetter_default = deburrLetter;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/deburr.js
var reLatin = /[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g;
var rsComboMarksRange = "\\u0300-\\u036f";
var reComboHalfMarksRange = "\\ufe20-\\ufe2f";
var rsComboSymbolsRange = "\\u20d0-\\u20ff";
var rsComboRange = rsComboMarksRange + reComboHalfMarksRange + rsComboSymbolsRange;
var rsCombo = "[" + rsComboRange + "]";
var reComboMark = RegExp(rsCombo, "g");
function deburr(string) {
  string = toString_default(string);
  return string && string.replace(reLatin, deburrLetter_default).replace(reComboMark, "");
}
var deburr_default = deburr;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_asciiWords.js
var reAsciiWord = /[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g;
function asciiWords(string) {
  return string.match(reAsciiWord) || [];
}
var asciiWords_default = asciiWords;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_hasUnicodeWord.js
var reHasUnicodeWord = /[a-z][A-Z]|[A-Z]{2}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/;
function hasUnicodeWord(string) {
  return reHasUnicodeWord.test(string);
}
var hasUnicodeWord_default = hasUnicodeWord;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_unicodeWords.js
var rsAstralRange = "\\ud800-\\udfff";
var rsComboMarksRange2 = "\\u0300-\\u036f";
var reComboHalfMarksRange2 = "\\ufe20-\\ufe2f";
var rsComboSymbolsRange2 = "\\u20d0-\\u20ff";
var rsComboRange2 = rsComboMarksRange2 + reComboHalfMarksRange2 + rsComboSymbolsRange2;
var rsDingbatRange = "\\u2700-\\u27bf";
var rsLowerRange = "a-z\\xdf-\\xf6\\xf8-\\xff";
var rsMathOpRange = "\\xac\\xb1\\xd7\\xf7";
var rsNonCharRange = "\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf";
var rsPunctuationRange = "\\u2000-\\u206f";
var rsSpaceRange = " \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000";
var rsUpperRange = "A-Z\\xc0-\\xd6\\xd8-\\xde";
var rsVarRange = "\\ufe0e\\ufe0f";
var rsBreakRange = rsMathOpRange + rsNonCharRange + rsPunctuationRange + rsSpaceRange;
var rsApos = "['\u2019]";
var rsBreak = "[" + rsBreakRange + "]";
var rsCombo2 = "[" + rsComboRange2 + "]";
var rsDigits = "\\d+";
var rsDingbat = "[" + rsDingbatRange + "]";
var rsLower = "[" + rsLowerRange + "]";
var rsMisc = "[^" + rsAstralRange + rsBreakRange + rsDigits + rsDingbatRange + rsLowerRange + rsUpperRange + "]";
var rsFitz = "\\ud83c[\\udffb-\\udfff]";
var rsModifier = "(?:" + rsCombo2 + "|" + rsFitz + ")";
var rsNonAstral = "[^" + rsAstralRange + "]";
var rsRegional = "(?:\\ud83c[\\udde6-\\uddff]){2}";
var rsSurrPair = "[\\ud800-\\udbff][\\udc00-\\udfff]";
var rsUpper = "[" + rsUpperRange + "]";
var rsZWJ = "\\u200d";
var rsMiscLower = "(?:" + rsLower + "|" + rsMisc + ")";
var rsMiscUpper = "(?:" + rsUpper + "|" + rsMisc + ")";
var rsOptContrLower = "(?:" + rsApos + "(?:d|ll|m|re|s|t|ve))?";
var rsOptContrUpper = "(?:" + rsApos + "(?:D|LL|M|RE|S|T|VE))?";
var reOptMod = rsModifier + "?";
var rsOptVar = "[" + rsVarRange + "]?";
var rsOptJoin = "(?:" + rsZWJ + "(?:" + [rsNonAstral, rsRegional, rsSurrPair].join("|") + ")" + rsOptVar + reOptMod + ")*";
var rsOrdLower = "\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])";
var rsOrdUpper = "\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])";
var rsSeq = rsOptVar + reOptMod + rsOptJoin;
var rsEmoji = "(?:" + [rsDingbat, rsRegional, rsSurrPair].join("|") + ")" + rsSeq;
var reUnicodeWord = RegExp([
  rsUpper + "?" + rsLower + "+" + rsOptContrLower + "(?=" + [rsBreak, rsUpper, "$"].join("|") + ")",
  rsMiscUpper + "+" + rsOptContrUpper + "(?=" + [rsBreak, rsUpper + rsMiscLower, "$"].join("|") + ")",
  rsUpper + "?" + rsMiscLower + "+" + rsOptContrLower,
  rsUpper + "+" + rsOptContrUpper,
  rsOrdUpper,
  rsOrdLower,
  rsDigits,
  rsEmoji
].join("|"), "g");
function unicodeWords(string) {
  return string.match(reUnicodeWord) || [];
}
var unicodeWords_default = unicodeWords;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/words.js
function words(string, pattern, guard) {
  string = toString_default(string);
  pattern = guard ? void 0 : pattern;
  if (pattern === void 0) {
    return hasUnicodeWord_default(string) ? unicodeWords_default(string) : asciiWords_default(string);
  }
  return string.match(pattern) || [];
}
var words_default = words;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/_createCompounder.js
var rsApos2 = "['\u2019]";
var reApos = RegExp(rsApos2, "g");
function createCompounder(callback) {
  return function(string) {
    return arrayReduce_default(words_default(deburr_default(string).replace(reApos, "")), callback, "");
  };
}
var createCompounder_default = createCompounder;

// ../../node_modules/.pnpm/lodash-es@4.17.21/node_modules/lodash-es/snakeCase.js
var snakeCase = createCompounder_default(function(result, word, index) {
  return result + (index ? "_" : "") + word.toLowerCase();
});
var snakeCase_default = snakeCase;

// src/helpers/convert-case.ts
function convertKeyCaseDeep(input, convertKey) {
  if (!isObject_default(input)) {
    return input;
  }
  if (Array.isArray(input)) {
    return input.map(
      (item) => convertKeyCaseDeep(item, convertKey)
    );
  }
  const output = {};
  for (const [key, value] of Object.entries(input)) {
    output[convertKey(key)] = convertKeyCaseDeep(value, convertKey);
  }
  return output;
}
function snakeCaseDeep(input) {
  return convertKeyCaseDeep(input, snakeCase_default);
}

// src/components/embeds/embed.tsx
function Embed(props) {
  return /* @__PURE__ */ React6.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedNode(props) }, props.children);
}
var EmbedNode = class extends Node {
  modifyMessageOptions(options) {
    const embed = {
      ...snakeCaseDeep(omit(this.props, ["children", "timestamp"])),
      timestamp: this.props.timestamp ? new Date(this.props.timestamp).toISOString() : void 0
    };
    for (const child of this.children) {
      if (child instanceof EmbedChildNode) {
        child.modifyEmbedOptions(embed);
      }
      if (child instanceof TextNode) {
        embed.description = (embed.description || "") + child.props;
      }
    }
    options.embeds.push(embed);
  }
};

// src/components/embeds/embed-author.tsx
import React7 from "react";
function EmbedAuthor(props) {
  return /* @__PURE__ */ React7.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedAuthorNode(props) }, /* @__PURE__ */ React7.createElement(DiscordJSReactElement, { props: {}, createNode: () => new AuthorTextNode({}) }, props.name ?? props.children));
}
var EmbedAuthorNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    var _a;
    options.author = {
      name: ((_a = this.children.findType(AuthorTextNode)) == null ? void 0 : _a.text) ?? "",
      url: this.props.url,
      icon_url: this.props.iconUrl
    };
  }
};
var AuthorTextNode = class extends Node {
};

// src/components/embeds/embed-field.tsx
import React8 from "react";
function EmbedField(props) {
  return /* @__PURE__ */ React8.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedFieldNode(props) }, /* @__PURE__ */ React8.createElement(DiscordJSReactElement, { props: {}, createNode: () => new FieldNameNode({}) }, props.name), /* @__PURE__ */ React8.createElement(DiscordJSReactElement, { props: {}, createNode: () => new FieldValueNode({}) }, props.value || props.children));
}
var EmbedFieldNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    var _a, _b;
    options.fields ??= [];
    options.fields.push({
      name: ((_a = this.children.findType(FieldNameNode)) == null ? void 0 : _a.text) ?? "",
      value: ((_b = this.children.findType(FieldValueNode)) == null ? void 0 : _b.text) ?? "",
      inline: this.props.inline
    });
  }
};
var FieldNameNode = class extends Node {
};
var FieldValueNode = class extends Node {
};

// src/components/embeds/embed-footer.tsx
import React9 from "react";
function EmbedFooter({ text, children, ...props }) {
  return /* @__PURE__ */ React9.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedFooterNode(props) }, /* @__PURE__ */ React9.createElement(DiscordJSReactElement, { props: {}, createNode: () => new FooterTextNode({}) }, text ?? children));
}
var EmbedFooterNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    var _a;
    options.footer = {
      text: ((_a = this.children.findType(FooterTextNode)) == null ? void 0 : _a.text) ?? "",
      icon_url: this.props.iconUrl
    };
    options.timestamp = this.props.timestamp ? new Date(this.props.timestamp).toISOString() : void 0;
  }
};
var FooterTextNode = class extends Node {
};

// src/components/embeds/embed-image.tsx
import React10 from "react";
function EmbedImage(props) {
  return /* @__PURE__ */ React10.createElement(
    DiscordJSReactElement,
    {
      props,
      createNode: () => new EmbedImageNode(props)
    }
  );
}
var EmbedImageNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    options.image = { url: this.props.url };
  }
};

// src/components/embeds/embed-thumbnail.tsx
import React11 from "react";
function EmbedThumbnail(props) {
  return /* @__PURE__ */ React11.createElement(
    DiscordJSReactElement,
    {
      props,
      createNode: () => new EmbedThumbnailNode(props)
    }
  );
}
var EmbedThumbnailNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    options.thumbnail = { url: this.props.url };
  }
};

// src/components/embeds/embed-title.tsx
import React12 from "react";
function EmbedTitle({ children, ...props }) {
  return /* @__PURE__ */ React12.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedTitleNode(props) }, /* @__PURE__ */ React12.createElement(DiscordJSReactElement, { props: {}, createNode: () => new TitleTextNode({}) }, children));
}
var EmbedTitleNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    var _a;
    options.title = ((_a = this.children.findType(
      TitleTextNode
    )) == null ? void 0 : _a.text) ?? "";
    options.url = this.props.url;
  }
};
var TitleTextNode = class extends Node {
};

// src/components/link.tsx
import { ButtonStyle as ButtonStyle2, ComponentType as ComponentType3 } from "discord.js";
import React13 from "react";
function Link({ label, children, ...props }) {
  return /* @__PURE__ */ React13.createElement(DiscordJSReactElement, { props, createNode: () => new LinkNode(props) }, /* @__PURE__ */ React13.createElement(DiscordJSReactElement, { props: {}, createNode: () => new LinkTextNode({}) }, label || children));
}
var LinkNode = class extends Node {
  getActionRowItemData() {
    var _a;
    return {
      type: ComponentType3.Button,
      style: ButtonStyle2.Link,
      disabled: this.props.disabled,
      emoji: this.props.emoji,
      label: ((_a = this.children.findType(LinkTextNode)) == null ? void 0 : _a.text) ?? "",
      url: this.props.url
    };
  }
  modifyMessageOptions(options) {
    getNextActionRow(options).push(
      this.getActionRowItemData()
    );
  }
};
var LinkTextNode = class extends Node {
};

// src/components/option.tsx
import React14 from "react";

// src/components/option-node.ts
var OptionNode = class extends Node {
  get options() {
    var _a, _b;
    return {
      label: ((_a = this.children.findType(OptionLabelNode)) == null ? void 0 : _a.text) ?? "",
      value: this.props.value,
      description: ((_b = this.children.findType(OptionDescriptionNode)) == null ? void 0 : _b.text) ?? "",
      emoji: this.props.emoji
    };
  }
};
var OptionLabelNode = class extends Node {
};
var OptionDescriptionNode = class extends Node {
};

// src/components/option.tsx
function Option({
  label,
  children,
  description,
  ...props
}) {
  return /* @__PURE__ */ React14.createElement(DiscordJSReactElement, { props, createNode: () => new OptionNode(props) }, (label !== void 0 || children !== void 0) && /* @__PURE__ */ React14.createElement(DiscordJSReactElement, { props: {}, createNode: () => new OptionLabelNode({}) }, label || children), description !== void 0 && /* @__PURE__ */ React14.createElement(
    DiscordJSReactElement,
    {
      props: {},
      createNode: () => new OptionDescriptionNode({})
    },
    description
  ));
}

// src/components/select.tsx
import { ComponentType as ComponentType4 } from "discord.js";
import { randomUUID as randomUUID3 } from "node:crypto";
import React15 from "react";
function Select(props) {
  return /* @__PURE__ */ React15.createElement(DiscordJSReactElement, { props, createNode: () => new SelectNode(props) }, props.children);
}
var SelectNode = class extends Node {
  customId = randomUUID3();
  getActionRowItemData() {
    const {
      multiple,
      value,
      values,
      minValues = 0,
      maxValues = 25,
      children,
      onChange,
      onChangeValue,
      onChangeMultiple,
      ...props
    } = this.props;
    const options = [...this.children].filter(
      isInstanceOf(OptionNode)
    ).map((node) => ({
      ...node.options,
      default: value === node.options.value && !this.isDeferred
    }));
    const item = {
      ...props,
      type: ComponentType4.StringSelect,
      customId: this.customId,
      options,
      disabled: this.props.disabled || this.isDeferred !== void 0,
      placeholder: this.isDeferred ? "Applying selection..." : props.placeholder
    };
    if (multiple) {
      item.minValues = minValues;
      item.maxValues = maxValues;
    }
    if (!multiple && value != void 0) {
    }
    return item;
  }
  modifyMessageOptions(message) {
    const actionRow = [];
    message.actionRows.push(actionRow);
    actionRow.push(
      this.getActionRowItemData()
    );
  }
  handleComponentInteraction(interaction, onComplete) {
    var _a, _b, _c, _d, _e, _f;
    const isSelectInteraction = interaction.isStringSelectMenu() && interaction.customId === this.customId && !this.props.disabled;
    if (!isSelectInteraction)
      return void 0;
    this.interaction = interaction;
    this.onComplete = onComplete;
    const firstValue = interaction.values.at(0);
    Promise.all([
      (_b = (_a = this.props).onChangeMultiple) == null ? void 0 : _b.call(_a, interaction.values, interaction),
      firstValue ? (_d = (_c = this.props).onChangeValue) == null ? void 0 : _d.call(_c, firstValue, interaction) : void 0,
      (_f = (_e = this.props).onChange) == null ? void 0 : _f.call(_e, interaction)
    ]).then(
      () => this.completeInteraction()
    ).catch(
      (error) => {
        throw error;
      }
    );
    return this;
  }
};

// src/test-message.ts
var TestDiscordJSReactMessage = class extends DiscordJSReactMessage {
  constructor(raw, renderer) {
    super(raw, renderer);
    this.raw = raw;
    this.renderer = renderer;
  }
  findButtonByLabel(label) {
    return this.renderer.findButtonByLabel(label);
  }
  findSelectByPlaceholder(placeholder) {
    return this.renderer.findSelectByPlaceholder(placeholder);
  }
  findLinkByLabel(label) {
    return this.renderer.findLinkByLabel(label);
  }
};

// src/test-renderer.ts
import { Events as Events2 } from "discord.js";
import { emitEvent, mockButtonInteraction, mockStringSelectInteraction } from "@answeroverflow/discordjs-mock";
var TestRenderer = class extends Renderer {
  trackMessage(message) {
    this.message = new TestDiscordJSReactMessage(message, this);
  }
  findButtonByLabel(label) {
    for (const node of this.nodes) {
      if (!(node instanceof ButtonNode))
        continue;
      if (node.props.label !== label)
        continue;
      return {
        ...node.getActionRowItemData(),
        click: async ({
          clicker,
          waitForInteractionToComplete = true
        }) => {
          if (!this.message)
            throw new Error("Message not found");
          const interaction = mockButtonInteraction({
            message: this.message.raw,
            caller: clicker,
            override: {
              custom_id: node.customId
            }
          });
          await emitEvent(this.discordJSReact.client, Events2.InteractionCreate, interaction);
          if (waitForInteractionToComplete) {
            await new Promise(
              (resolve) => {
                const interval = setInterval(() => {
                  if (interaction.replied) {
                    clearInterval(interval);
                    resolve(void 0);
                  }
                }, 100);
              }
            );
          }
        }
      };
    }
  }
  findLinkByLabel(label) {
    for (const node of this.nodes) {
      if (!(node instanceof LinkNode))
        continue;
      if (node.props.label !== label)
        continue;
      return node.getActionRowItemData();
    }
  }
  findSelectByPlaceholder(placeholder) {
    for (const node of this.nodes) {
      if (!(node instanceof SelectNode)) {
        continue;
      }
      if (node.props.placeholder !== placeholder) {
        continue;
      }
      return {
        ...node.getActionRowItemData(),
        select: async ({
          clicker,
          waitForInteractionToComplete = true,
          values
        }) => {
          if (!this.message)
            throw new Error("Message not found");
          const interaction = mockStringSelectInteraction({
            message: this.message.raw,
            caller: clicker,
            data: {
              custom_id: node.customId,
              values
            }
          });
          await emitEvent(this.discordJSReact.client, Events2.InteractionCreate, interaction);
          if (waitForInteractionToComplete) {
            await new Promise(
              (resolve) => {
                const interval = setInterval(() => {
                  if (interaction.replied) {
                    clearInterval(interval);
                    resolve(void 0);
                  }
                }, 100);
              }
            );
          }
        }
      };
    }
  }
};

// src/test-discordjs-react.tsx
import React16 from "react";
var TestDiscordJSReact = class extends DiscordJSReact {
  constructor(client, config2 = {
    wrapper: ({ children }) => /* @__PURE__ */ React16.createElement(React16.Fragment, null, children)
  }) {
    super(client, config2);
    this.client = client;
    this.config = config2;
  }
  createRenderer(renderer, initialContent) {
    return new TestRenderer(renderer, this, initialContent);
  }
};
export {
  ActionRow2 as ActionRow,
  Button,
  ButtonNode,
  DiscordJSReact,
  Embed,
  EmbedAuthor,
  EmbedField,
  EmbedFooter,
  EmbedImage,
  EmbedThumbnail,
  EmbedTitle,
  Link,
  LinkNode,
  Option,
  Select,
  SelectNode,
  TestDiscordJSReact,
  TestDiscordJSReactMessage,
  TestRenderer,
  useInstance
};
/*! Bundled license information:

lodash-es/lodash.js:
  (**
   * @license
   * Lodash (Custom Build) <https://lodash.com/>
   * Build: `lodash modularize exports="es" -o ./`
   * Copyright OpenJS Foundation and other contributors <https://openjsf.org/>
   * Released under MIT license <https://lodash.com/license>
   * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
   * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
   *)
*/
//# sourceMappingURL=index.js.map